@@ -13,50 +13,33 @@
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
-# License along with this program; if not, write to the
-# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
-# Boston, MA 02110-1301, USA.
+# License along with this program; if not, see <http://www.gnu.org/licenses/>.
 import re
 from gettext import gettext as _
 
 from gi.repository import Gdk
 from gi.repository import GES
 from gi.repository import Gio
-from gi.repository import GObject
 from gi.repository import Gtk
 
 from pitivi.timeline import elements
 from pitivi.undo.timeline import CommitTimelineFinalizingAction
-from pitivi.utils import ui
 from pitivi.utils.loggable import Loggable
 from pitivi.utils.timeline import Zoomable
+from pitivi.utils.ui import LAYER_HEIGHT
+from pitivi.utils.ui import PADDING
+from pitivi.utils.ui import SEPARATOR_HEIGHT
 
 
-class TwoStateButton(Gtk.Button):
-    """Button with two states and according labels/images."""
+AUDIO_ICONS = {
+    True: "audio-volume-high-symbolic",
+    False: "audio-volume-muted-symbolic",
+}
 
-    __gsignals__ = {
-        "changed-state": (GObject.SignalFlags.RUN_LAST, None, (GObject.TYPE_PYOBJECT,),)
-    }
-
-    def __init__(self, state1="", state2=""):
-        Gtk.Button.__init__(self)
-        self.set_relief(Gtk.ReliefStyle.NONE)
-        self.connect("clicked", self._clickedCb)
-
-        self.set_states(state1, state2)
-        self._state = True
-
-        self.set_label(self.states[self._state])
-
-    def set_states(self, state1, state2):
-        self.states = {True: state1, False: state2}
-
-    def _clickedCb(self, unused_widget):
-        self._state = not self._state
-
-        self.set_label(self.states[self._state])
-        self.emit("changed-state", self._state)
+VIDEO_ICONS = {
+    True: "eye-open-negative-filled-symbolic",
+    False: "eye-not-looking-symbolic",
+}
 
 
 class SpacedSeparator(Gtk.EventBox):
@@ -65,40 +48,15 @@ class SpacedSeparator(Gtk.EventBox):
     Inherits from EventBox since we want to change background color.
     """
 
-    def __init__(self, position):
+    def __init__(self):
         Gtk.EventBox.__init__(self)
 
-        self.box = Gtk.Box()
-        self.box.set_orientation(Gtk.Orientation.VERTICAL)
-        self.add(self.box)
-        self.__position = position
-
         self.get_style_context().add_class("SpacedSeparator")
-        self.box.get_style_context().add_class("SpacedSeparator")
-        self._update()
-
-    def do_state_flags_changed(self, old_flags):
-        self._update()
-        Gtk.EventBox.do_state_flags_changed(self, old_flags)
-
-    def _update(self):
-        HIGLIGHTED_PADDING = 3
-        total_height = ui.PADDING + HIGLIGHTED_PADDING
-        if not self.get_state_flags() & Gtk.StateFlags.PRELIGHT:
-            self.props.height_request = 1
-            self.props.margin_bottom = (total_height - 1) / 2
-            self.props.margin_top = (total_height - 1) / 2
-        else:
-            self.props.height_request = ui.PADDING
-            if self.__position == Gtk.PositionType.TOP:
-                self.props.margin_bottom = HIGLIGHTED_PADDING
-                self.props.margin_top = 0
-            else:
-                self.props.margin_bottom = 0
-                self.props.margin_top = HIGLIGHTED_PADDING
+        self.props.height_request = SEPARATOR_HEIGHT
 
 
 class LayerControls(Gtk.EventBox, Loggable):
+    """Container with widgets for controlling a layer."""
 
     __gtype_name__ = 'PitiviLayerControls'
 
@@ -109,12 +67,11 @@ class LayerControls(Gtk.EventBox, Loggable):
         self.ges_layer = ges_layer
         self.ges_timeline = self.ges_layer.get_timeline()
         self.app = app
+        self.__icon = None
 
-        self.props.width_request = ui.CONTROL_WIDTH
-        # Half the height because we display only the video strip when empty.
-        self.props.height_request = ui.LAYER_HEIGHT / 2 + ui.PADDING * 3
-        self.props.hexpand = True
-        self.props.valign = Gtk.Align.FILL
+        tracks = self.ges_timeline.get_tracks()
+        self.timeline_audio_tracks = [track for track in tracks if track.props.track_type == GES.TrackType.AUDIO]
+        self.timeline_video_tracks = [track for track in tracks if track.props.track_type == GES.TrackType.VIDEO]
 
         hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
         self.add(hbox)
@@ -122,94 +79,99 @@ class LayerControls(Gtk.EventBox, Loggable):
         vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         hbox.pack_start(vbox, True, True, 0)
 
-        self.sep = Gtk.Separator.new(Gtk.Orientation.VERTICAL)
-        self.sep.props.margin_top = ui.PADDING / 2
-        self.sep.props.margin_bottom = ui.PADDING / 2
-        hbox.pack_start(self.sep, False, False, 0)
-
-        self.before_sep = SpacedSeparator(Gtk.PositionType.TOP)
-        self.before_sep.props.width_request = self.props.width_request - 1
-        self.before_sep.props.vexpand = False
-        vbox.pack_start(self.before_sep, False, False, 0)
+        rightside_separator = Gtk.Separator.new(Gtk.Orientation.VERTICAL)
+        hbox.pack_start(rightside_separator, False, False, 0)
 
         name_row = Gtk.Box()
         name_row.set_orientation(Gtk.Orientation.HORIZONTAL)
-        name_row.props.spacing = ui.PADDING
-        name_row.props.margin_left = ui.PADDING
-        name_row.props.margin_right = ui.PADDING
+        name_row.props.margin_top = PADDING
+        name_row.props.margin_left = PADDING
+        name_row.props.margin_right = PADDING
         vbox.pack_start(name_row, False, False, 0)
 
-        self.name_entry = Gtk.Entry()
-        self.name_entry.get_style_context().add_class("LayerControlEntry")
-        self.name_entry.props.valign = Gtk.Align.CENTER
-        self.name_entry.connect("focus-out-event", self.__name_focus_out_cb)
-        self.ges_layer.connect("notify-meta", self.__layer_rename_cb)
-        self.__updateName()
-        name_row.pack_start(self.name_entry, True, True, 0)
-
         self.menubutton = Gtk.MenuButton.new()
         self.menubutton.props.valign = Gtk.Align.CENTER
         self.menubutton.props.relief = Gtk.ReliefStyle.NONE
-        model, action_group = self.__createMenuModel()
+        model, action_group = self.__create_menu_model()
         popover = Gtk.Popover.new_from_model(self.menubutton, model)
         popover.insert_action_group("layer", action_group)
         popover.props.position = Gtk.PositionType.LEFT
         self.menubutton.set_popover(popover)
         name_row.pack_start(self.menubutton, False, False, 0)
 
+        self.name_entry = Gtk.Entry()
+        self.name_entry.get_style_context().add_class("LayerControlEntry")
+        self.name_entry.props.valign = Gtk.Align.CENTER
+        self.name_entry.connect("focus-out-event", self.__name_focus_out_cb)
+        self.ges_layer.connect("notify-meta", self.__layer_rename_cb)
+        self.__update_name()
+        name_row.pack_start(self.name_entry, True, True, 0)
+
+        self.audio_button = Gtk.Button.new()
+        self.audio_button.connect("clicked", self.__audio_button_clicked_cb)
+        self.__update_audio_button()
+
+        self.video_button = Gtk.Button.new()
+        self.video_button.connect("clicked", self.__video_button_clicked_cb)
+        self.__update_video_button()
+
+        control_box = Gtk.ButtonBox()
+        control_box.set_layout(Gtk.ButtonBoxStyle.EXPAND)
+        control_box.add(self.video_button)
+        control_box.add(self.audio_button)
+        name_row.pack_start(control_box, False, False, 0)
+
         space = Gtk.Label()
         space.props.vexpand = True
         vbox.pack_start(space, False, False, 0)
 
-        self.after_sep = SpacedSeparator(Gtk.PositionType.BOTTOM)
-        self.after_sep.props.width_request = self.props.width_request - 1
-        self.after_sep.props.vexpand = False
-        vbox.pack_start(self.after_sep, False, False, 0)
-
-        self.ges_layer.connect("notify::priority", self.__layerPriorityChangedCb)
-        self.ges_timeline.connect("layer-added", self.__timelineLayerAddedCb)
-        self.ges_timeline.connect("layer-removed", self.__timelineLayerRemovedCb)
-        self.__updateActions()
+        self.ges_layer.connect("notify::priority", self.__layer_priority_changed_cb)
+        self.ges_layer.connect("active-changed", self.__layer_active_changed_cb)
+        self.ges_timeline.connect("layer-added", self.__timeline_layer_added_cb)
+        self.ges_timeline.connect("layer-removed", self.__timeline_layer_removed_cb)
+        self.__update_actions()
 
         # When the window property is set, specify the mouse cursor.
-        self.connect("notify::window", self.__windowSetCb)
+        self.connect("notify::window", self.__window_set_cb)
 
-    def __windowSetCb(self, unused_window, unused_pspec):
+    def __window_set_cb(self, unused_window, unused_pspec):
         self.props.window.set_cursor(Gdk.Cursor.new(Gdk.CursorType.HAND1))
 
     def __del__(self):
         self.name_entry.disconnect_by_func(self.__name_focus_out_cb)
         self.ges_layer.disconnect_by_func(self.__layer_rename_cb)
-        self.ges_layer.disconnect_by_func(self.__layerPriorityChangedCb)
-        self.ges_timeline.disconnect_by_func(self.__timelineLayerAddedCb)
-        self.ges_timeline.disconnect_by_func(self.__timelineLayerRemovedCb)
+        self.ges_layer.disconnect_by_func(self.__layer_priority_changed_cb)
+        self.ges_timeline.disconnect_by_func(self.__timeline_layer_added_cb)
+        self.ges_timeline.disconnect_by_func(self.__timeline_layer_removed_cb)
         super().__del__()
 
     def __layer_rename_cb(self, unused_ges_layer, item, value):
         if not item == "video::name":
             return
-        self.__updateName()
+        self.__update_name()
 
     def __name_focus_out_cb(self, unused_widget, unused_event):
-        current_name = self.ges_layer.ui.getName()
+        self.name_entry.delete_selection()
+        current_name = self.ges_layer.ui.get_name()
         name = self.name_entry.get_text()
         if name == current_name:
             return
 
-        with self.app.action_log.started("change layer name"):
-            self.ges_layer.ui.setName(name)
+        with self.app.action_log.started("change layer name",
+                                         toplevel=True):
+            self.ges_layer.ui.set_name(name)
 
-    def __layerPriorityChangedCb(self, unused_ges_layer, unused_pspec):
-        self.__updateActions()
-        self.__updateName()
+    def __layer_priority_changed_cb(self, unused_ges_layer, unused_pspec):
+        self.__update_actions()
+        self.__update_name()
 
-    def __timelineLayerAddedCb(self, unused_timeline, unused_ges_layer):
-        self.__updateActions()
+    def __timeline_layer_added_cb(self, unused_timeline, unused_ges_layer):
+        self.__update_actions()
 
-    def __timelineLayerRemovedCb(self, unused_timeline, unused_ges_layer):
-        self.__updateActions()
+    def __timeline_layer_removed_cb(self, unused_timeline, unused_ges_layer):
+        self.__update_actions()
 
-    def __updateActions(self):
+    def __update_actions(self):
         priority = self.ges_layer.get_priority()
         first = priority == 0
         self.__move_layer_up_action.props.enabled = not first
@@ -218,12 +180,12 @@ class LayerControls(Gtk.EventBox, Loggable):
         last = priority == layers_count - 1
         self.__move_layer_down_action.props.enabled = not last
         self.__move_layer_bottom_action.props.enabled = not last
-        self.__delete_layer_action.props.enabled = layers_count > 1
+        self.delete_layer_action.props.enabled = layers_count > 1
 
-    def __updateName(self):
-        self.name_entry.set_text(self.ges_layer.ui.getName())
+    def __update_name(self):
+        self.name_entry.set_text(self.ges_layer.ui.get_name())
 
-    def __createMenuModel(self):
+    def __create_menu_model(self):
         action_group = Gio.SimpleActionGroup()
         menu_model = Gio.Menu()
 
@@ -251,8 +213,8 @@ class LayerControls(Gtk.EventBox, Loggable):
         action_group.add_action(action)
         menu_model.append(_("Move layer to bottom"), "layer.%s" % action.get_name().replace(" ", "."))
 
-        self.__delete_layer_action = Gio.SimpleAction.new("delete-layer", None)
-        action = self.__delete_layer_action
+        self.delete_layer_action = Gio.SimpleAction.new("delete-layer", None)
+        action = self.delete_layer_action
         action.connect("activate", self.__delete_layer_cb)
         action_group.add_action(action)
         menu_model.append(_("Delete layer"), "layer.%s" % action.get_name())
@@ -262,9 +224,13 @@ class LayerControls(Gtk.EventBox, Loggable):
     def __delete_layer_cb(self, unused_action, unused_parameter):
         pipeline = self.ges_timeline.get_asset().pipeline
         with self.app.action_log.started("delete layer",
-                                         CommitTimelineFinalizingAction(pipeline)):
+                                         finalizing_action=CommitTimelineFinalizingAction(pipeline),
+                                         toplevel=True):
             self.ges_timeline.remove_layer(self.ges_layer)
-        pipeline.commit_timeline()
+            removed_priority = self.ges_layer.props.priority
+            for ges_layer in self.ges_timeline.get_layers():
+                if ges_layer.props.priority > removed_priority:
+                    ges_layer.props.priority -= 1
 
     def __move_layer_cb(self, unused_simple_action, unused_parameter, step):
         index = self.ges_layer.get_priority()
@@ -274,86 +240,65 @@ class LayerControls(Gtk.EventBox, Loggable):
             index = 0
         else:
             index = len(self.ges_timeline.get_layers()) - 1
-        self.ges_timeline.ui.moveLayer(self.ges_layer, index)
+        self.ges_timeline.ui.move_layer(self.ges_layer, index)
         self.app.project_manager.current_project.pipeline.commit_timeline()
 
+    def __audio_button_clicked_cb(self, button):
+        self.ges_layer.set_active_for_tracks(not self.__check_tracks_active(
+            self.timeline_audio_tracks), self.timeline_audio_tracks)
+        self.app.project_manager.current_project.pipeline.commit_timeline()
+
+    def __update_audio_button(self):
+        active = self.__check_tracks_active(self.timeline_audio_tracks)
+        icon = AUDIO_ICONS[active]
+        self.audio_button.set_image(Gtk.Image.new_from_icon_name(icon, Gtk.IconSize.BUTTON))
+
+    def __video_button_clicked_cb(self, button):
+        self.ges_layer.set_active_for_tracks(not self.__check_tracks_active(
+            self.timeline_video_tracks), self.timeline_video_tracks)
+        self.app.project_manager.current_project.pipeline.commit_timeline()
+
+    def __update_video_button(self):
+        active = self.__check_tracks_active(self.timeline_video_tracks)
+        icon = VIDEO_ICONS[active]
+        self.video_button.set_image(Gtk.Image.new_from_icon_name(icon, Gtk.IconSize.BUTTON))
+
+    def __layer_active_changed_cb(self, ges_layer, active, tracks):
+        self.__update_video_button()
+        self.__update_audio_button()
+
+    def __check_tracks_active(self, tracks):
+        return all([self.ges_layer.get_active_for_track(t) for t in tracks])
+
     def update(self, media_types):
-        self.props.height_request = self.ges_layer.ui.props.height_request + ui.PADDING * 3
+        self.props.height_request = self.ges_layer.ui.props.height_request
 
-        if media_types & GES.TrackType.VIDEO:
-            icon = "video-x-generic"
+        has_audio = media_types & GES.TrackType.AUDIO
+        self.audio_button.set_sensitive(has_audio)
+
+        has_video = media_types & GES.TrackType.VIDEO
+        self.video_button.set_sensitive(has_video)
+
+        if has_video or not media_types:
+            # The layer has video or is empty.
+            icon = "video-x-generic-symbolic"
         else:
-            icon = "audio-x-generic"
-        image = Gtk.Image.new_from_icon_name(icon, Gtk.IconSize.BUTTON)
-        self.menubutton.props.image = image
+            # The layer has audio and nothing else.
+            icon = "audio-x-generic-symbolic"
 
-        # TODO: Use media_types to determine which controls to show.
+        if icon != self.__icon:
+            image = Gtk.Image.new_from_icon_name(icon, Gtk.IconSize.BUTTON)
+            self.menubutton.props.image = image
+            self.__icon = icon
 
 
-class LayerLayout(Gtk.Layout, Loggable):
-    """Displays the clips of a layer."""
-
-    __gtype_name__ = "PitiviLayerLayout"
-
-    def __init__(self, timeline):
-        Gtk.Layout.__init__(self)
-        Loggable.__init__(self)
-
-        self._children = []
-        self._changed = False
-        self.timeline = timeline
-
-        self.props.hexpand = True
-        self.get_style_context().add_class("LayerLayout")
-
-    def do_add(self, widget):
-        self._children.append(widget)
-        self._children.sort(key=lambda clip: clip.z_order)
-        Gtk.Layout.do_add(self, widget)
-        self._changed = True
-
-        for child in self._children:
-            if isinstance(child, elements.TransitionClip):
-                window = child.get_window()
-                if window is not None:
-                    window.raise_()
-
-    def do_remove(self, widget):
-        self._children.remove(widget)
-        self._changed = True
-        Gtk.Layout.do_remove(self, widget)
-
-    def put(self, child, x, y):
-        self._children.append(child)
-        self._children.sort(key=lambda clip: clip.z_order)
-        Gtk.Layout.put(self, child, x, y)
-        self._changed = True
-
-    def do_draw(self, cr):
-        if self._changed:
-            self._children.sort(key=lambda clip: clip.z_order)
-            for child in self._children:
-
-                if isinstance(child, elements.TransitionClip):
-                    window = child.get_window()
-                    window.raise_()
-            self._changed = False
-
-        self.props.width = max(self.timeline.layout.get_allocation().width,
-                               Zoomable.nsToPixel(self.timeline.ges_timeline.props.duration))
-        self.props.width_request = max(self.timeline.layout.get_allocation().width,
-                                       Zoomable.nsToPixel(self.timeline.ges_timeline.props.duration))
-
-        for child in self._children:
-            self.propagate_draw(child, cr)
-
-
-class Layer(Gtk.EventBox, Zoomable, Loggable):
+class Layer(Gtk.Layout, Zoomable, Loggable):
+    """Container for the clips widgets of a layer."""
 
     __gtype_name__ = "PitiviLayer"
 
     def __init__(self, ges_layer, timeline):
-        Gtk.EventBox.__init__(self)
+        Gtk.Layout.__init__(self)
         Zoomable.__init__(self)
         Loggable.__init__(self)
 
@@ -362,54 +307,51 @@ class Layer(Gtk.EventBox, Zoomable, Loggable):
         self.timeline = timeline
         self.app = timeline.app
 
-        self.ges_layer.connect("clip-added", self._clipAddedCb)
-        self.ges_layer.connect("clip-removed", self._clipRemovedCb)
+        self._children = []
+        self._changed = False
 
-        # FIXME Make the layer height user setable with 'Paned'
-        self.props.height_request = ui.LAYER_HEIGHT / 2
+        self.ges_layer.connect("clip-added", self._clip_added_cb)
+        self.ges_layer.connect("clip-removed", self._clip_removed_cb)
+
+        # The layer is always the width of the Timeline which contains it.
+        self.props.hexpand = True
         self.props.valign = Gtk.Align.START
 
-        self._layout = LayerLayout(self.timeline)
-        self._layout.connect("remove", self.__childWidgetRemovedCb)
-        self.add(self._layout)
-
         self.media_types = GES.TrackType(0)
-        for clip in ges_layer.get_clips():
-            self._addClip(clip)
+        for ges_clip in ges_layer.get_clips():
+            self._add_clip(ges_clip)
+        self.check_media_types()
 
-        self.before_sep = SpacedSeparator(Gtk.PositionType.TOP)
-        self.after_sep = SpacedSeparator(Gtk.PositionType.BOTTOM)
-
-    def setName(self, name):
+    def set_name(self, name):
         self.ges_layer.set_meta("video::name", name)
 
-    def _nameIfSet(self):
+    def _name_if_set(self):
         name = self.ges_layer.get_meta("video::name")
         if not name:
             name = self.ges_layer.get_meta("audio::name")
         return name
 
-    def __nameIfMeaningful(self):
-        name = self._nameIfSet()
+    def __name_if_meaningful(self):
+        name = self._name_if_set()
         if name:
             for pattern in ("video [0-9]+$", "audio [0-9]+$", "Layer [0-9]+$"):
                 if re.match(pattern, name):
                     return None
         return name
 
-    def getName(self):
-        name = self.__nameIfMeaningful()
+    def get_name(self):
+        name = self.__name_if_meaningful()
         if not name:
             name = _('Layer %d') % self.ges_layer.get_priority()
         return name
 
     def release(self):
-        for clip in self.ges_layer.get_clips():
-            self._removeClip(clip)
-        self.ges_layer.disconnect_by_func(self._clipAddedCb)
-        self.ges_layer.disconnect_by_func(self._clipRemovedCb)
+        for ges_clip in self.ges_layer.get_clips():
+            self._remove_clip(ges_clip)
+        self.ges_layer.disconnect_by_func(self._clip_added_cb)
+        self.ges_layer.disconnect_by_func(self._clip_removed_cb)
 
-    def checkMediaTypes(self):
+    def check_media_types(self):
         if self.timeline.editing_context:
             self.info("Not updating media types as"
                       " we are editing the timeline")
@@ -427,53 +369,50 @@ class Layer(Gtk.EventBox, Zoomable, Loggable):
                     # Cannot find more types than these.
                     break
 
-        if not (self.media_types & GES.TrackType.AUDIO) and not (self.media_types & GES.TrackType.VIDEO):
-            # An empty layer only shows the video strip.
-            self.media_types = GES.TrackType.VIDEO
+        if (self.media_types & GES.TrackType.AUDIO) and (self.media_types & GES.TrackType.VIDEO):
+            self.props.height_request = LAYER_HEIGHT
+        else:
+            # If the layer is empty, set layer's height to default height.
+            self.props.height_request = LAYER_HEIGHT / 2
 
-        height = 0
-        if self.media_types & GES.TrackType.AUDIO:
-            height += ui.LAYER_HEIGHT / 2
-        if self.media_types & GES.TrackType.VIDEO:
-            height += ui.LAYER_HEIGHT / 2
-
-        self.props.height_request = height
         if hasattr(self.ges_layer, "control_ui") and self.ges_layer.control_ui:
             self.ges_layer.control_ui.update(self.media_types)
 
         if old_media_types != self.media_types:
-            self.updatePosition()
+            self.update_position()
 
-    def move(self, child, x, y):
-        self._layout.move(child, x, y)
+    def _clip_child_added_cb(self, ges_clip, child):
+        self.check_media_types()
 
-    def _childAddedToClipCb(self, ges_clip, child):
-        self.checkMediaTypes()
+    def _clip_child_removed_cb(self, ges_clip, child):
+        self.check_media_types()
 
-    def _childRemovedFromClipCb(self, ges_clip, child):
-        self.checkMediaTypes()
+    def _clip_added_cb(self, unused_ges_layer, ges_clip):
+        self._add_clip(ges_clip)
+        self.check_media_types()
 
-    def _clipAddedCb(self, layer, ges_clip):
-        self._addClip(ges_clip)
-
-    def _addClip(self, ges_clip):
+    def _add_clip(self, ges_clip):
         ui_type = elements.GES_TYPE_UI_TYPE.get(ges_clip.__gtype__, None)
         if ui_type is None:
             self.error("Implement UI for type %s?", ges_clip.__gtype__)
             return
 
-        clip = ui_type(self, ges_clip)
+        widget = ui_type(self, ges_clip)
+        self._children.append(widget)
+        self._children.sort(key=lambda clip: clip.z_order)
+        self.put(widget, self.ns_to_pixel(ges_clip.props.start), 0)
+        widget.update_position()
+        self._changed = True
+        widget.show_all()
 
-        self._layout.put(clip, self.nsToPixel(ges_clip.props.start), 0)
-        self.show_all()
-        ges_clip.connect_after("child-added", self._childAddedToClipCb)
-        ges_clip.connect_after("child-removed", self._childRemovedFromClipCb)
-        self.checkMediaTypes()
+        ges_clip.connect_after("child-added", self._clip_child_added_cb)
+        ges_clip.connect_after("child-removed", self._clip_child_removed_cb)
 
-    def _clipRemovedCb(self, ges_layer, ges_clip):
-        self._removeClip(ges_clip)
+    def _clip_removed_cb(self, unused_ges_layer, ges_clip):
+        self._remove_clip(ges_clip)
+        self.check_media_types()
 
-    def _removeClip(self, ges_clip):
+    def _remove_clip(self, ges_clip):
         if not ges_clip.ui:
             return
 
@@ -482,21 +421,28 @@ class Layer(Gtk.EventBox, Zoomable, Loggable):
             self.error("Implement UI for type %s?", ges_clip.__gtype__)
             return
 
+        self.remove(ges_clip.ui)
+        self._children.remove(ges_clip.ui)
+        self._changed = True
         ges_clip.ui.release()
-        self._layout.remove(ges_clip.ui)
-        self.timeline.selection.unselect([ges_clip])
+        ges_clip.ui = None
 
-    def __childWidgetRemovedCb(self, layout, clip):
-        ges_clip = clip.ges_clip
-        if self.timeline.draggingElement is None:
-            ges_clip.ui.release()
-            ges_clip.ui = None
+        ges_clip.disconnect_by_func(self._clip_child_added_cb)
+        ges_clip.disconnect_by_func(self._clip_child_removed_cb)
 
-        ges_clip.disconnect_by_func(self._childAddedToClipCb)
-        ges_clip.disconnect_by_func(self._childRemovedFromClipCb)
-        self.checkMediaTypes()
-
-    def updatePosition(self):
+    def update_position(self):
         for ges_clip in self.ges_layer.get_clips():
             if hasattr(ges_clip, "ui"):
-                ges_clip.ui.updatePosition()
+                ges_clip.ui.update_position()
+
+    def do_draw(self, cr):
+        if self._changed:
+            self._children.sort(key=lambda clip: clip.z_order)
+            for child in self._children:
+                if isinstance(child, elements.TransitionClip):
+                    window = child.get_window()
+                    window.raise_()
+            self._changed = False
+
+        for child in self._children:
+            self.propagate_draw(child, cr)
