@@ -14,7 +14,7 @@
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this program; if not, see <http://www.gnu.org/licenses/>.
-import collections
+import collections.abc
 import errno
 import fnmatch
 import os
@@ -478,8 +478,7 @@ def do_log(level, obj, category, message, args, where=-1, file_path=None, line=N
             try:
                 handler(level, obj, category, file_path, line, message)
             except TypeError as e:
-                raise SystemError("handler %r raised a TypeError: %s" % (
-                    handler, get_exception_message(e)))
+                raise SystemError("handler %r unusable" % handler) from e
 
     return ret
 
@@ -714,7 +713,7 @@ def add_log_handler(func):
     Raises:
         TypeError: When func is not a callable.
     """
-    if not isinstance(func, collections.Callable):
+    if not isinstance(func, collections.abc.Callable):
         raise TypeError("func must be callable")
 
     if func not in _log_handlers:
@@ -736,7 +735,7 @@ def add_limited_log_handler(func):
     Raises:
         TypeError: When func is not a callable.
     """
-    if not isinstance(func, collections.Callable):
+    if not isinstance(func, collections.abc.Callable):
         raise TypeError("func must be callable")
 
     if func not in _log_handlers_limited:
